define(["exports", "./utils/scheduler.js", "./ResizeObservation.js", "./ResizeObserverDetail.js", "./utils/resizeObservers.js"], function (_exports, _scheduler, _ResizeObservation, _ResizeObserverDetail, _resizeObservers) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ResizeObserverController = void 0;
  var observerMap = new Map();

  var getObservationIndex = function getObservationIndex(observationTargets, target) {
    for (var i = 0; i < observationTargets.length; i += 1) {
      if (observationTargets[i].target === target) {
        return i;
      }
    }

    return -1;
  };

  var ResizeObserverController = function () {
    function ResizeObserverController() {}

    ResizeObserverController.connect = function (resizeObserver, callback) {
      var detail = new _ResizeObserverDetail.ResizeObserverDetail(resizeObserver, callback);

      _resizeObservers.resizeObservers.push(detail);

      observerMap.set(resizeObserver, detail);
    };

    ResizeObserverController.observe = function (resizeObserver, target, options) {
      if (observerMap.has(resizeObserver)) {
        var detail = observerMap.get(resizeObserver);

        if (getObservationIndex(detail.observationTargets, target) < 0) {
          detail.observationTargets.push(new _ResizeObservation.ResizeObservation(target, options && options.box));
          (0, _scheduler.updateCount)(1);

          _scheduler.scheduler.schedule();
        }
      }
    };

    ResizeObserverController.unobserve = function (resizeObserver, target) {
      if (observerMap.has(resizeObserver)) {
        var detail = observerMap.get(resizeObserver);
        var index = getObservationIndex(detail.observationTargets, target);

        if (index >= 0) {
          detail.observationTargets.splice(index, 1);
          (0, _scheduler.updateCount)(-1);
        }
      }
    };

    ResizeObserverController.disconnect = function (resizeObserver) {
      if (observerMap.has(resizeObserver)) {
        var detail = observerMap.get(resizeObserver);

        _resizeObservers.resizeObservers.splice(_resizeObservers.resizeObservers.indexOf(detail), 1);

        observerMap.delete(resizeObserver);
        (0, _scheduler.updateCount)(-detail.observationTargets.length);
      }
    };

    return ResizeObserverController;
  }();

  _exports.ResizeObserverController = ResizeObserverController;
});