define(["../iron-resizable-behavior/iron-resizable-behavior.js", "../polymer/lib/legacy/polymer-fn.js", "../polymer/lib/legacy/polymer.dom.js", "../polymer/lib/utils/html-tag.js", "../polymer/polymer-legacy.js"], function (_ironResizableBehavior, _polymerFn, _polymerDom, _htmlTag, _polymerLegacy) {
  "use strict";

  function _templateObject_1f88c970c07b11ea85f79731808de288() {
    var data = babelHelpers.taggedTemplateLiteral(["\n    <style>\n      :host {\n        display: block;\n        transition-duration: var(--iron-collapse-transition-duration, 300ms);\n        /* Safari 10 needs this property prefixed to correctly apply the custom property */\n        -webkit-transition-duration: var(--iron-collapse-transition-duration, 300ms);\n        overflow: visible;\n      }\n\n      :host(.iron-collapse-closed) {\n        display: none;\n      }\n\n      :host(:not(.iron-collapse-opened)) {\n        overflow: hidden;\n      }\n    </style>\n\n    <slot></slot>\n"]);

    _templateObject_1f88c970c07b11ea85f79731808de288 = function _templateObject_1f88c970c07b11ea85f79731808de288() {
      return data;
    };

    return data;
  }

  /**
  `iron-collapse` creates a collapsible block of content.  By default, the content
  will be collapsed.  Use `opened` or `toggle()` to show/hide the content.
  
      <button on-click="toggle">toggle collapse</button>
  
      <iron-collapse id="collapse">
        <div>Content goes here...</div>
      </iron-collapse>
  
      ...
  
      toggle: function() {
        this.$.collapse.toggle();
      }
  
  `iron-collapse` adjusts the max-height/max-width of the collapsible element to
  show/hide the content.  So avoid putting padding/margin/border on the
  collapsible directly, and instead put a div inside and style that.
  
      <style>
        .collapse-content {
          padding: 15px;
          border: 1px solid #dedede;
        }
      </style>
  
      <iron-collapse>
        <div class="collapse-content">
          <div>Content goes here...</div>
        </div>
      </iron-collapse>
  
  ### Styling
  
  The following custom properties and mixins are available for styling:
  
  Custom property | Description | Default
  ----------------|-------------|----------
  `--iron-collapse-transition-duration` | Animation transition duration | `300ms`
  
  @group Iron Elements
  @hero hero.svg
  @demo demo/index.html
  @element iron-collapse
  */
  (0, _polymerFn.Polymer)({
    _template: (0, _htmlTag.html)(_templateObject_1f88c970c07b11ea85f79731808de288()),
    is: 'iron-collapse',
    behaviors: [_ironResizableBehavior.IronResizableBehavior],
    properties: {
      /**
       * If true, the orientation is horizontal; otherwise is vertical.
       *
       * @attribute horizontal
       */
      horizontal: {
        type: Boolean,
        value: false,
        observer: '_horizontalChanged'
      },

      /**
       * Set opened to true to show the collapse element and to false to hide it.
       *
       * @attribute opened
       */
      opened: {
        type: Boolean,
        value: false,
        notify: true,
        observer: '_openedChanged'
      },

      /**
       * When true, the element is transitioning its opened state. When false,
       * the element has finished opening/closing.
       *
       * @attribute transitioning
       */
      transitioning: {
        type: Boolean,
        notify: true,
        readOnly: true
      },

      /**
       * Set noAnimation to true to disable animations.
       *
       * @attribute noAnimation
       */
      noAnimation: {
        type: Boolean
      },

      /**
       * Stores the desired size of the collapse body.
       * @private
       */
      _desiredSize: {
        type: String,
        value: ''
      }
    },

    get dimension() {
      return this.horizontal ? 'width' : 'height';
    },

    /**
     * `maxWidth` or `maxHeight`.
     * @private
     */
    get _dimensionMax() {
      return this.horizontal ? 'maxWidth' : 'maxHeight';
    },

    /**
     * `max-width` or `max-height`.
     * @private
     */
    get _dimensionMaxCss() {
      return this.horizontal ? 'max-width' : 'max-height';
    },

    hostAttributes: {
      role: 'group',
      'aria-hidden': 'true'
    },
    listeners: {
      transitionend: '_onTransitionEnd'
    },

    /**
     * Toggle the opened state.
     *
     * @method toggle
     */
    toggle: function toggle() {
      this.opened = !this.opened;
    },
    show: function show() {
      this.opened = true;
    },
    hide: function hide() {
      this.opened = false;
    },

    /**
     * Updates the size of the element.
     * @param {string} size The new value for `maxWidth`/`maxHeight` as css property value, usually `auto` or `0px`.
     * @param {boolean=} animated if `true` updates the size with an animation, otherwise without.
     */
    updateSize: function updateSize(size, animated) {
      // Consider 'auto' as '', to take full size.
      size = size === 'auto' ? '' : size;
      var willAnimate = animated && !this.noAnimation && this.isAttached && this._desiredSize !== size;
      this._desiredSize = size;

      this._updateTransition(false); // If we can animate, must do some prep work.


      if (willAnimate) {
        // Animation will start at the current size.
        var startSize = this._calcSize(); // For `auto` we must calculate what is the final size for the animation.
        // After the transition is done, _transitionEnd will set the size back to
        // `auto`.


        if (size === '') {
          this.style[this._dimensionMax] = '';
          size = this._calcSize();
        } // Go to startSize without animation.


        this.style[this._dimensionMax] = startSize; // Force layout to ensure transition will go. Set scrollTop to itself
        // so that compilers won't remove it.

        this.scrollTop = this.scrollTop; // Enable animation.

        this._updateTransition(true); // If final size is the same as startSize it will not animate.


        willAnimate = size !== startSize;
      } // Set the final size.


      this.style[this._dimensionMax] = size; // If it won't animate, call transitionEnd to set correct classes.

      if (!willAnimate) {
        this._transitionEnd();
      }
    },

    /**
     * enableTransition() is deprecated, but left over so it doesn't break
     * existing code. Please use `noAnimation` property instead.
     *
     * @method enableTransition
     * @deprecated since version 1.0.4
     */
    enableTransition: function enableTransition(enabled) {
      _polymerLegacy.Base._warn('`enableTransition()` is deprecated, use `noAnimation` instead.');

      this.noAnimation = !enabled;
    },
    _updateTransition: function _updateTransition(enabled) {
      this.style.transitionDuration = enabled && !this.noAnimation ? '' : '0s';
    },
    _horizontalChanged: function _horizontalChanged() {
      this.style.transitionProperty = this._dimensionMaxCss;
      var otherDimension = this._dimensionMax === 'maxWidth' ? 'maxHeight' : 'maxWidth';
      this.style[otherDimension] = '';
      this.updateSize(this.opened ? 'auto' : '0px', false);
    },
    _openedChanged: function _openedChanged() {
      this.setAttribute('aria-hidden', !this.opened);

      this._setTransitioning(true);

      this.toggleClass('iron-collapse-closed', false);
      this.toggleClass('iron-collapse-opened', false);
      this.updateSize(this.opened ? 'auto' : '0px', true); // Focus the current collapse.

      if (this.opened) {
        this.focus();
      }
    },
    _transitionEnd: function _transitionEnd() {
      this.style[this._dimensionMax] = this._desiredSize;
      this.toggleClass('iron-collapse-closed', !this.opened);
      this.toggleClass('iron-collapse-opened', this.opened);

      this._updateTransition(false);

      this.notifyResize();

      this._setTransitioning(false);
    },
    _onTransitionEnd: function _onTransitionEnd(event) {
      if ((0, _polymerDom.dom)(event).rootTarget === this) {
        this._transitionEnd();
      }
    },
    _calcSize: function _calcSize() {
      return this.getBoundingClientRect()[this.dimension] + 'px';
    }
  });
});