define(["exports", "../lit-html.js"], function (_exports, _litHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.ifDefined = void 0;

  /**
   * @license
   * Copyright (c) 2018 The Polymer Project Authors. All rights reserved.
   * This code may only be used under the BSD style license found at
   * http://polymer.github.io/LICENSE.txt
   * The complete set of authors may be found at
   * http://polymer.github.io/AUTHORS.txt
   * The complete set of contributors may be found at
   * http://polymer.github.io/CONTRIBUTORS.txt
   * Code distributed by Google as part of the polymer project is also
   * subject to an additional IP rights grant found at
   * http://polymer.github.io/PATENTS.txt
   */
  var previousValues = new WeakMap();
  /**
   * For AttributeParts, sets the attribute if the value is defined and removes
   * the attribute if the value is undefined.
   *
   * For other part types, this directive is a no-op.
   */

  var ifDefined = (0, _litHtml.directive)(function (value) {
    return function (part) {
      var previousValue = previousValues.get(part);

      if (value === undefined && babelHelpers.instanceof(part, _litHtml.AttributePart)) {
        // If the value is undefined, remove the attribute, but only if the value
        // was previously defined.
        if (previousValue !== undefined || !previousValues.has(part)) {
          var name = part.committer.name;
          part.committer.element.removeAttribute(name);
        }
      } else if (value !== previousValue) {
        part.setValue(value);
      }

      previousValues.set(part, value);
    };
  });
  _exports.ifDefined = ifDefined;
});