define(["exports", "../lib/parts.js", "../lit-html.js"], function (_exports, _parts, _litHtml) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.unsafeHTML = void 0;

  /**
   * @license
   * Copyright (c) 2017 The Polymer Project Authors. All rights reserved.
   * This code may only be used under the BSD style license found at
   * http://polymer.github.io/LICENSE.txt
   * The complete set of authors may be found at
   * http://polymer.github.io/AUTHORS.txt
   * The complete set of contributors may be found at
   * http://polymer.github.io/CONTRIBUTORS.txt
   * Code distributed by Google as part of the polymer project is also
   * subject to an additional IP rights grant found at
   * http://polymer.github.io/PATENTS.txt
   */
  // For each part, remember the value that was last rendered to the part by the
  // unsafeHTML directive, and the DocumentFragment that was last set as a value.
  // The DocumentFragment is used as a unique key to check if the last value
  // rendered to the part was with unsafeHTML. If not, we'll always re-render the
  // value passed to unsafeHTML.
  var previousValues = new WeakMap();
  /**
   * Renders the result as HTML, rather than text.
   *
   * Note, this is unsafe to use with any user-provided input that hasn't been
   * sanitized or escaped, as it may lead to cross-site-scripting
   * vulnerabilities.
   */

  var unsafeHTML = (0, _litHtml.directive)(function (value) {
    return function (part) {
      if (!babelHelpers.instanceof(part, _litHtml.NodePart)) {
        throw new Error('unsafeHTML can only be used in text bindings');
      }

      var previousValue = previousValues.get(part);

      if (previousValue !== undefined && (0, _parts.isPrimitive)(value) && value === previousValue.value && part.value === previousValue.fragment) {
        return;
      }

      var template = document.createElement('template');
      template.innerHTML = value; // innerHTML casts to string internally

      var fragment = document.importNode(template.content, true);
      part.setValue(fragment);
      previousValues.set(part, {
        value: value,
        fragment: fragment
      });
    };
  });
  _exports.unsafeHTML = unsafeHTML;
});