define(["exports"], function (_exports) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.isReplacedElement = _exports.isElement = _exports.isHidden = _exports.isSVG = void 0;

  var isSVG = function (target) {
    return target instanceof SVGElement && 'getBBox' in target;
  };

  _exports.isSVG = isSVG;

  var isHidden = function (target) {
    if (isSVG(target)) {
      var _a = target.getBBox(),
          width = _a.width,
          height = _a.height;

      return !width && !height;
    }

    var _b = target,
        offsetWidth = _b.offsetWidth,
        offsetHeight = _b.offsetHeight;
    return !(offsetWidth || offsetHeight || target.getClientRects().length);
  };

  _exports.isHidden = isHidden;

  var isElement = function (obj) {
    var _a, _b;

    var scope = (_b = (_a = obj) === null || _a === void 0 ? void 0 : _a.ownerDocument) === null || _b === void 0 ? void 0 : _b.defaultView;
    return !!(scope && obj instanceof scope.Element);
  };

  _exports.isElement = isElement;

  var isReplacedElement = function (target) {
    switch (target.tagName) {
      case 'INPUT':
        if (target.type !== 'image') {
          break;
        }

      case 'VIDEO':
      case 'AUDIO':
      case 'EMBED':
      case 'OBJECT':
      case 'CANVAS':
      case 'IFRAME':
      case 'IMG':
        return true;
    }

    return false;
  };

  _exports.isReplacedElement = isReplacedElement;
});