define(["exports", "../../../@polymer/polymer/polymer-element.js", "../../../@polymer/polymer/lib/elements/custom-style.js", "./vaadin-text-field.js"], function (_exports, _polymerElement, _customStyle, _vaadinTextField) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.EmailFieldElement = void 0;

  /**
  @license
  Copyright (c) 2018 Vaadin Ltd.
  This program is available under Apache License Version 2.0, available at https://vaadin.com/license/
  */

  /**
   * `<vaadin-email-field>` is a Web Component for email field control in forms.
   *
   * ```html
   * <vaadin-email-field label="Email">
   * </vaadin-email-field>
   * ```
   *
   * ### Styling
   *
   * See vaadin-text-field.html for the styling documentation
   *
   * See [ThemableMixin – how to apply styles for shadow parts](https://github.com/vaadin/vaadin-themable-mixin/wiki)
   *
   * @memberof Vaadin
   * @demo demo/index.html
   */
  class EmailFieldElement extends _vaadinTextField.TextFieldElement {
    static get is() {
      return 'vaadin-email-field';
    }

    static get version() {
      return '2.5.4';
    }

    ready() {
      super.ready();
      this.inputElement.type = 'email';
      this.inputElement.autocapitalize = 'off';
    }

    _createConstraintsObserver() {
      // NOTE: pattern needs to be set before constraints observer is initialized
      this.pattern = '^[a-zA-Z0-9.!#$%&’*+/=?^_`{|}~-]+@[a-zA-Z0-9-]+(?:\.[a-zA-Z0-9-]+)*$';

      super._createConstraintsObserver();
    }

  }

  _exports.EmailFieldElement = EmailFieldElement;
  customElements.define(EmailFieldElement.is, EmailFieldElement);
});