define(["./chart-styles.js", "../../@polymer/polymer/polymer-element.js", "../../@polymer/iron-resizable-behavior/iron-resizable-behavior.js", "../../@polymer/polymer/lib/legacy/class.js", "./resize-behavior.js", "./context-behavior.js", "./chart-property-behavior.js"], function (_chartStyles, _polymerElement, _ironResizableBehavior, _class, _resizeBehavior, _contextBehavior, _chartPropertyBehavior) {
  "use strict";

  /**
  A radar chart is a way of showing multiple data points and the variation between them.
  
  They are often useful for comparing the points of two or more different data sets.
  
  ##### Example
  
      <chart-radar data="[[data]]"></chart-radar>
  
      ...
  
      this.data = {
        labels: ["Eating", "Drinking", "Sleeping", "Designing", "Coding", "Cycling", "Running"],
        datasets: [
            {
              label: "My First dataset",
              backgroundColor: "rgba(179,181,198,0.2)",
              borderColor: "rgba(179,181,198,1)",
              pointBackgroundColor: "rgba(179,181,198,1)",
              pointBorderColor: "#fff",
              pointHoverBackgroundColor: "#fff",
              pointHoverBorderColor: "rgba(179,181,198,1)",
              data: [65, 59, 90, 81, 56, 55, 40]
            },
            {
              label: "My Second dataset",
              backgroundColor: "rgba(255,99,132,0.2)",
              borderColor: "rgba(255,99,132,1)",
              pointBackgroundColor: "rgba(255,99,132,1)",
              pointBorderColor: "#fff",
              pointHoverBackgroundColor: "#fff",
              pointHoverBorderColor: "rgba(255,99,132,1)",
              data: [28, 48, 40, 19, 96, 27, 100]
            }
          ]
        };
  
  @group Chart Elements
  @element chart-radar
  @demo demo/chart-radar.html
  */

  /*
  Chart JS needs to be imported somewhere. Importing here creates errors for IE11 and Chrome 41.
  //import 'chart.js/dist/Chart.js';
  */
  class ChartRadar extends (0, _resizeBehavior.ChartResizeBehavior)((0, _contextBehavior.ChartContextBehavior)((0, _chartPropertyBehavior.ChartPropertyBehavior)((0, _class.mixinBehaviors)([_ironResizableBehavior.IronResizableBehavior], _polymerElement.PolymerElement)))) {
    static get template() {
      return (0, _polymerElement.html)`
    <style include="chart-styles"></style>

    <div>
      <canvas id="canvas"></canvas>
    </div>
`;
    }

    static get is() {
      return 'chart-radar';
    }

    ready() {
      super.ready();

      this._setType('radar');
    }

  }

  window.customElements.define(ChartRadar.is, ChartRadar);
});